-- Daily Global Check
-- by Jadya
-- EU-Well of Eternity

-- [[ DGC v1.2 Multi Character Selection ]]--

local addonName, addonTable = ...
local foreach = table.foreach
local tx_cross = "Interface\\RaidFrame\\ReadyCheck-NotReady"

local L = addonTable.L
local Region = GetCurrentRegion() --GetCVar("portal")

local seconds_per_hour = 60 * 60
local seconds_per_day = seconds_per_hour * 24

local chars = DailyGlobalCheck.CharFunctions

local pID = addonTable.pID

function chars:isquestcompleted(questID, character)
 local c = DailyGlobalCheck_CharData[character]
 if not c or not c.data or not c.data[questID] then return end

 if chars:isOutDated(c.data[questID][1], c.resets) then 
  --print("DGC DEBUG: quest "..questID.." is outdated - chars:isquestcompleted == false")
  c.data[questID] = nil
  return false
 else
  return c.data[questID], c.data[questID][2]
 end
end

function chars:PackQuest(qData, cstring)
 local result = { qData and qData[7] or nil, cstring}
 return result
end

local function calculateResets()

 local weekly_reset_time = { hour = 8, min = 0 } -- not 100% sure.. but it should be maintenance time
 local weekly_reset_day = Region == 1 and 3 or 4

 local t = time()
 local currDate = date("*t")
 local weeklyReset = t
 local dailyReset = t + GetQuestResetTime()

 local wd = currDate.wday

 local i = currDate.hour

 while i ~= weekly_reset_time.hour do
  if i < weekly_reset_time.hour then
   weeklyReset = weeklyReset + seconds_per_hour
   i = i + 1
  else
   weeklyReset = weeklyReset - seconds_per_hour
   i = i - 1
  end
 end
 
 -- subtract the remaining minutes to reach 8:00
 weeklyReset = weeklyReset - currDate.min * 60
 
 i = wd
 while i ~= weekly_reset_day do
  i = i + 1
  weeklyReset = weeklyReset + seconds_per_day
  if i > 7 then i = 1 end
 end
 
 return weeklyReset, dailyReset
end

function chars:isOutDated(qType, t)
 -- normal quests are never outdated
 if qType == "Q" then return false end
 
 local now = time()
 local dReset = t.dReset
 local wReset = t.wReset

 local isWeekly = qType == "W"

 if isWeekly then
  return now > wReset
 else 
  return now > dReset
 end
end

function chars:SaveCharData()

 if tContains(DailyGlobalCheck_Options["chars_blacklist"], pID) then
  return
 end
 
 local result = {}
 result.data = {}
 
 -- Custom List
  foreach(DGC_CustomLists["Main"]["Order"], function(_, o)
    foreach(o, function(_, questIDs)
     foreach(questIDs, function(index, qID)	 
      if index > 1 then
 	   local b, cstring = addonTable.isquestcompleted(qID, true, v)
	   if b then
        result.data[qID] = chars:PackQuest(DGC_CustomLists["Main"]["Data"][qID], cstring)
	   end
	  end
     end)
    end)
   end)

 foreach(addonTable.Plugins, function(k,v)
  if v.MultiCharsEnabled then
   foreach(v["Order"], function(_, o)
    foreach(o, function(_, questIDs)
     foreach(questIDs, function(index, qID)	 
      if index > 1 then
 	   local b, cstring = addonTable.isquestcompleted(qID, true, v)
	   if b then
        result.data[qID] = chars:PackQuest(v["Data"][qID], cstring)
	   end
	  end
     end)
    end)
   end)
  end
 end)
 
 local wr, dr = calculateResets()
 result.resets = { wReset = wr, dReset = dr }
 
 --[[ debug
 local tmp = date("*t", wr)
 print("weekly reset")
 print(tmp.day.."/"..tmp.month.."/"..tmp.year.." ("..tmp.wday..") - "..tmp.hour..":"..tmp.min)
 
 result.debug = { w = tmp }
 
 tmp = date("*t", dr)
 print("daily reset")
 print(tmp.day.."/"..tmp.month.."/"..tmp.year.." ("..tmp.wday..") - "..tmp.hour..":"..tmp.min)
 
 --result.debug.d = tmp
 ]]
 
 DailyGlobalCheck_CharData[pID] = result
end

local frames_backdrop  = { bgFile = "Interface\\Buttons\\GreyscaleRamp64", 
                           edgeFile = "Interface\\Buttons\\WHITE8X8", 
						   edgeSize = 1 }

function chars:CreateCharactersFrame(parent)

 DailyGlobalCheck.CharactersFrame = CreateFrame("Frame", "DGCCharactersFrame", parent)
 local f = DailyGlobalCheck.CharactersFrame

  local function repositionbuttons()
   local w,h,counter = 300,0,0
   foreach(f.buttons, function(k,v)
	v:SetPoint("TOPLEFT", f, "TOPLEFT", 2, -(2 + counter * 20))
    counter = counter + 1
    h = h + 20
   end)
   f:SetSize(w,h)
  end
 
  local function btnselect(self)
   if DailyGlobalCheck.selectedpID == self.pID then return end
   
   foreach(f.buttons, function(_,v) v:SetBackdropColor(0.1,0.1,0.1,1) end)
   self:SetBackdropColor(0,0.3,0,1)
   DailyGlobalCheck.selectedpID = self.pID
   addonTable.refresh()
  end

  local function deleteChar(btn)
   local i = -1
   foreach(f.buttons, function(k,v) if v == btn then i = k return end end)
   if i > -1 then
    table.remove(f.buttons, i)
	if DailyGlobalCheck_CharData[btn.pID] then DailyGlobalCheck_CharData[btn.pID] = nil end
	btn:Hide()
	repositionbuttons()
	if not tContains(DailyGlobalCheck_Options["chars_blacklist"], btn.pID) then
	 table.insert(DailyGlobalCheck_Options["chars_blacklist"], btn.pID)
	end
	print(string.format(L["chars_deleted"], btn.pID))
   end
  end
 
  local function createCharButton(pid, isPlayer)
   if (not isPlayer) and (pid == pID or
      tContains(DailyGlobalCheck_Options["chars_blacklist"], pid)) then return end
   
   local btn = CreateFrame("Frame", nil, f)
 
   btn:SetBackdrop(frames_backdrop)
   btn:SetBackdropColor(0.1,0.1,0.1,1)
   btn:SetBackdropBorderColor(0.2,0.2,0.2,1)
   btn:SetFrameStrata("MEDIUM")
   btn:SetSize(300, 20)
   
   btn.text = btn:CreateFontString(nil, "OVERLAY", "GameTooltipText")
   btn.text:SetAllPoints()
   btn.text:SetText(pid)

   btn:SetScript("OnEnter", function()
    btn:SetBackdropBorderColor(0,0.8,0.8,1)
   end)
   btn:SetScript("OnLeave", function()
    btn:SetBackdropBorderColor(0.2,0.2,0.2,1)
   end)
   btn:SetScript("OnMouseUp", btnselect)
   
   btn.pID = pid
   
   if DailyGlobalCheck.selectedpID == pid then
    btn:SetBackdropColor(0,0.3,0,1)
   end
   
   if pid ~= pID then
    local delbtn = CreateFrame("Frame", nil, btn)
    delbtn:SetSize(20,20)
    delbtn:SetPoint("TOPRIGHT")
    delbtn.texture = delbtn:CreateTexture()
    delbtn.texture:SetAllPoints()
    delbtn.texture:SetTexture(tx_cross)
    delbtn:SetScript("OnMouseUp", function() deleteChar(btn) end)
   end
   
   table.insert(f.buttons, btn)
  end

 f.buttons = {}
 f:SetPoint("TOP", parent, "BOTTOM")
 f:SetBackdrop(frames_backdrop)
 f:SetBackdropColor(0.07,0.07,0.07,0.9)
 f:SetBackdropBorderColor(0.07,0.07,0.07,0.9)
 f:SetFrameStrata("MEDIUM")

 createCharButton(pID, true)
 foreach(DailyGlobalCheck_CharData, function(k) createCharButton(k) end)
 repositionbuttons()

 f:Hide()
 
 return f
end

DGCEventFrame:RegisterEvent("PLAYER_LEAVING_WORLD")
function DGCEventFrame.PLAYER_LEAVING_WORLD(...)
 chars:SaveCharData()
end